@@ -7,11 +7,14 @@ use POSIX qw/ceil floor/;
 use Scalar::Util qw/blessed looks_like_number/;
 require Exporter;
 
-our $VERSION = '0.010';
+our $VERSION = '0.011';
 our @ISA = 'Exporter';
 our @EXPORT_OK = 'calc';
 our $ERROR;
 
+# See disclaimer in Math::Round
+use constant ROUND_HALF => 0.50000000000008;
+
 {
 	my %operators = (
 		'<<' => { assoc => 'left' },
@@ -87,9 +90,16 @@ our $ERROR;
 		floor => { args => 1, code => sub { floor _real($_[0]) } },
 		ceil  => { args => 1, code => sub { ceil _real($_[0]) } },
 		rand  => { args => 0, code => sub { rand } },
+		# Adapted from Math::Round
+		round => { args => 1, code => sub { _real($_[0]) >= 0
+		                                    ? floor(_real($_[0]) + ROUND_HALF)
+		                                    : ceil(_real($_[0]) - ROUND_HALF) } },
 	);
 	
 	sub _default_functions { +{%functions} }
+	
+	my $singleton;
+	sub _instance { blessed $_[0] ? $_[0] : ($singleton //= $_[0]->new) }
 }
 
 sub new {
@@ -159,7 +169,7 @@ my $token_re = qr{(
 
 sub parse {
 	my ($self, $expr) = @_;
-	$self = $self->new unless ref $self;
+	$self = _instance($self);
 	my (@expr_queue, @oper_stack, $binop_possible);
 	while ($expr =~ /$token_re/g) {
 		my ($token, $octal) = ($1, $2);
@@ -210,9 +220,8 @@ sub parse {
 	
 	# Leftover operators go at the end
 	while (@oper_stack) {
-		die "Mismatched parentheses\n"
-			if $oper_stack[-1]{type} eq 'paren';
-		push @expr_queue, (pop @oper_stack)->{value};
+		die "Mismatched parentheses\n" if $oper_stack[-1] eq '(';
+		push @expr_queue, pop @oper_stack;
 	}
 	
 	return \@expr_queue;
@@ -228,22 +237,21 @@ sub _shunt_operator {
 	my ($expr_queue, $oper_stack, $oper) = @_;
 	my $oper_stat = _operator($oper);
 	my $assoc = $oper_stat->{assoc};
-	while (@$oper_stack and $oper_stack->[-1]{type} eq 'operator') {
-		my $top_oper = $oper_stack->[-1]{value};
+	while (@$oper_stack and defined _operator(my $top_oper = $oper_stack->[-1])) {
 		if ($oper_stat->{lower_than}{$top_oper}
 		    or ($assoc eq 'left' and $oper_stat->{equal_to}{$top_oper})) {
-			push @$expr_queue, (pop @$oper_stack)->{value};
+			push @$expr_queue, pop @$oper_stack;
 		} else {
 			last;
 		}
 	}
-	push @$oper_stack, { type => 'operator', value => $oper };
+	push @$oper_stack, $oper;
 	return 1;
 }
 
 sub _shunt_function_with_args {
 	my ($expr_queue, $oper_stack, $function) = @_;
-	push @$oper_stack, { type => 'function', value => $function };
+	push @$oper_stack, $function;
 	return 1;
 }
 
@@ -255,37 +263,39 @@ sub _shunt_function_no_args {
 
 sub _shunt_left_paren {
 	my ($expr_queue, $oper_stack) = @_;
-	push @$oper_stack, { type => 'paren', value => '(' };
+	push @$oper_stack, '(';
 	return 1;
 }
 
 sub _shunt_right_paren {
 	my ($expr_queue, $oper_stack) = @_;
-	while (@$oper_stack and $oper_stack->[-1]{type} ne 'paren') {
-		push @$expr_queue, (pop @$oper_stack)->{value};
+	while (@$oper_stack and $oper_stack->[-1] ne '(') {
+		push @$expr_queue, pop @$oper_stack;
 	}
-	return 0 unless @$oper_stack and $oper_stack->[-1]{type} eq 'paren';
+	return 0 unless @$oper_stack and $oper_stack->[-1] eq '(';
 	pop @$oper_stack;
-	if (@$oper_stack and $oper_stack->[-1]{type} eq 'function') {
-		push @$expr_queue, (pop @$oper_stack)->{value};
+	if (@$oper_stack and $oper_stack->[-1] ne '('
+	    and !defined _operator($oper_stack->[-1])) {
+		# Not parentheses or operator, must be function
+		push @$expr_queue, pop @$oper_stack;
 	}
 	return 1;
 }
 
 sub _shunt_comma {
 	my ($expr_queue, $oper_stack) = @_;
-	while (@$oper_stack and $oper_stack->[-1]{type} ne 'paren') {
-		push @$expr_queue, (pop @$oper_stack)->{value};
+	while (@$oper_stack and $oper_stack->[-1] ne '(') {
+		push @$expr_queue, pop @$oper_stack;
 	}
-	return 0 unless @$oper_stack and $oper_stack->[-1]{type} eq 'paren';
+	return 0 unless @$oper_stack and $oper_stack->[-1] eq '(';
 	return 1;
 }
 
-sub calc ($) { Math::Calc::Parser->evaluate($_[0]); }
+sub calc ($) { __PACKAGE__->evaluate($_[0]) }
 
 sub evaluate {
 	my ($self, $expr) = @_;
-	$self = $self->new unless ref $self;
+	$self = _instance($self);
 	$expr = $self->parse($expr) unless ref $expr eq 'ARRAY';
 	
 	die "No expression to evaluate\n" unless @$expr;
@@ -325,7 +335,7 @@ sub evaluate {
 
 sub try_evaluate {
 	my ($self, $expr) = @_;
-	$self = $self->new unless ref $self;
+	$self = _instance($self);
 	$self->clear_error;
 	undef $ERROR;
 	local $@;
@@ -552,6 +562,10 @@ Log with arbitrary base given as second argument.
 
 Random value between 0 and 1 (exclusive of 1).
 
+=item round
+
+Round to nearest integer, with halfway cases rounded away from zero.
+
 =item sin
 
 Sine.
@@ -575,6 +589,11 @@ Due to the nature of handling complex numbers, the evaluated result may be a
 L<Math::Complex> object. These objects can be directly printed or used in
 numeric operations but may be more difficult to use in comparisons.
 
+Operators and functions that are not defined to operate on complex numbers will
+return the result of the operation on the real components of their operands.
+This includes the operators C<E<lt>E<lt>>, C<E<gt>E<gt>>, and C<%>, and the
+functions C<int>, C<floor>, C<ceil>, and C<round>.
+
 =head1 BUGS
 
 Report any issues on the public bugtracker.
